package org.simplericity.serberuhs.filter;

import javax.servlet.FilterConfig;
import java.io.File;

/**
 */
public class DefaultFilterConfiguration implements KerberosFilterConfiguration {
    private static final String ENABLED_PROP = "enabled";
    private static final String KEYTAB_FILE_PROP = "keytabFile";
    private static final String PRINCIPAL_PROP = "principal";
    private static final String PASSWORD_PROP = "password";
    private static final String FALLBACK_LOGIN_PROP = "fallbackLogin";
    private final boolean enabled;
    private final File keytabFile;
    private final String principal;
    private final String password;
    private final FilterConfig filterConfig;

    public DefaultFilterConfiguration(FilterConfig filterConfig) {
        this.filterConfig = filterConfig;
        verifyConfigurationPresent(filterConfig);
        keytabFile = new File(filterConfig.getInitParameter(KEYTAB_FILE_PROP));
        if (!keytabFile.exists()) {
            throw new IllegalArgumentException(KEYTAB_FILE_PROP + " not found " + keytabFile.getAbsolutePath());
        }
        principal = filterConfig.getInitParameter(PRINCIPAL_PROP);
        password = filterConfig.getInitParameter(PASSWORD_PROP);
        enabled = ! "false".equals(filterConfig.getInitParameter(ENABLED_PROP));

    }

    private void verifyConfigurationPresent(FilterConfig filterConfig) {

        String[] props = new String[]{KEYTAB_FILE_PROP, PRINCIPAL_PROP, PASSWORD_PROP};

        for (String prop : props) {
            if (filterConfig.getInitParameter(prop) == null) {
                throw new IllegalArgumentException("Filter configuration parameter '" + prop + "' is required");
            }
        }
    }

    public File getKeytabFile() {
        return keytabFile;
    }


    public String getPrincipal() {
        return principal;
    }

    public String getPassword() {
        return password;
    }

    public String getFallbackLoginPath() {
        return filterConfig.getInitParameter(FALLBACK_LOGIN_PROP);
    }

    public boolean isEnabled() {
        return enabled;
    }
}
