package org.simplericity.serberuhs.filter;

import org.simplericity.serberuhs.DefaultKerberosSubjectFactory;
import org.simplericity.serberuhs.SpNego;
import org.simplericity.serberuhs.SpNegoResult;
import org.simplericity.serberuhs.KerberosSubjectFactory;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 */
public class KerberosFilter implements Filter {
    private KerberosSubjectFactory factory;
    private KerberosFilterConfiguration configuration;
    public  static final String AUTORIZED_PRINCIPAL_SESSION_ATTRIBUTE = KerberosFilter.class.getName() + "_AUTHORIZED_PRINCIPAL";


    public final void init(FilterConfig filterConfig) throws ServletException {
        KerberosSubjectFactory factory = new DefaultKerberosSubjectFactory();
        factory.setConfiguration(configuration = createConfiguration(filterConfig));
        this.factory = factory;
    }


    protected KerberosFilterConfiguration createConfiguration(FilterConfig filterConfig) {
        return new DefaultFilterConfiguration(filterConfig);
    }

    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        if(configuration.isEnabled()) {
            final HttpServletRequest req = (HttpServletRequest) request;
            final HttpServletResponse res = (HttpServletResponse) response;
            SpNego spNego = createSpNego();

            spNego.negotiate(factory.getSubject(), req, res);

            if(spNego.getResult() == SpNegoResult.AUTHORIZED) {
                handleSuccessfulAuthorization(spNego.getAuthorizedPrincipal(), req, res, chain);
            } else if(spNego.getResult() == SpNegoResult.MISSING_AUTHORIZATION_HEADER) {
                handleMissingAuthorizationHeader(req, res, chain);
            } else {
                handleUnsuccessfulAutorization(spNego, req, res, chain);
            }
        } else {
            chain.doFilter(request, response);
        }
    }

    protected SpNego createSpNego() {
        return new SpNego();
    }

    public void setFactory(KerberosSubjectFactory factory) {
        this.factory = factory;
    }

    public void setConfiguration(KerberosFilterConfiguration configuration) {
        this.configuration = configuration;
    }

    protected void dispatchToFallback(HttpServletRequest req, HttpServletResponse res) throws IOException, ServletException {
        if(configuration.getFallbackLoginPath() != null) {

            req.getRequestDispatcher(configuration.getFallbackLoginPath()).forward(req, res);


        }
    }

    protected void handleMissingAuthorizationHeader(HttpServletRequest req, HttpServletResponse res, FilterChain chain) throws IOException, ServletException {
        dispatchToFallback(req, res);
    }

    protected void handleUnsuccessfulAutorization(SpNego spNego, HttpServletRequest req, HttpServletResponse res, FilterChain chain) throws IOException, ServletException {
        dispatchToFallback(req, res);
    }

    protected void handleSuccessfulAuthorization(String authorizedPrincipal, HttpServletRequest req, HttpServletResponse res, FilterChain chain) throws IOException, ServletException {

        exposeSessionAttribute(authorizedPrincipal, req);

        chain.doFilter(req, res);
    }

    protected void exposeSessionAttribute(String authorizedPrincipal, HttpServletRequest req) {
        req.getSession().setAttribute(AUTORIZED_PRINCIPAL_SESSION_ATTRIBUTE, authorizedPrincipal);
    }


    public void destroy() {

    }



}
